#include "Sphere.hxx"

Sphere::Sphere(const Vec3f& center, float radius, Shader* shader)
  : Primitive(shader),
    m_center(center),
    m_radius(radius)
{
}

Sphere::~Sphere()
{
}

bool
Sphere::Intersect(Ray& ray)
{
  float A = ray.direction().dot(ray.direction());
  float C = (ray.origin()-m_center).dot(ray.origin()-m_center) - m_radius*m_radius;
  float B = 2 * ray.direction().dot(ray.origin()-m_center);

  if( B*B-4*A*C < 0 )
    return false;

  float root = sqrtf(B*B-4*A*C);
  float t = (-B-root)/(2.0f*A);
  if(t > ray.t())
    return false;

  if( t < 1e-6 )
    {
      t = (-B+root)/(2.0f*A);
      if( t < 1e-6 || t > ray.t())
	return false;
    }
  ray.setT(t);
  return true;
}

Vec3f
Sphere::GetNormal(Ray& ray)
{
  // We don't want to modify the ray (probably this is not needed, but I am
  // too lazy to think about it now ...)
  Ray tempRay = ray;

  // Surface normal is the difference between intersection and center point
  if(Intersect(tempRay)) {
    // intersection point
    Vec3f i = tempRay.origin() + tempRay.direction() * (tempRay.t() - Epsilon);
    // normal
    Vec3f n = (i - m_center);
    n.normalize();
    return n;
  } else {
    // no intersection with ray, so no surface normal
    return Vec3f(0,0,0);
  }
}
